package sun.util.resources;

import java.util.NoSuchElementException;
import java.util.HashSet;
import java.util.Iterator;
import java.util.AbstractSet;
import java.util.concurrent.ConcurrentHashMap;
import java.util.Collections;
import java.util.Enumeration;
import java.util.concurrent.atomic.AtomicMarkableReference;
import java.util.Set;
import java.util.concurrent.ConcurrentMap;
import java.util.ResourceBundle;

public abstract class ParallelListResourceBundle extends ResourceBundle
{
    private volatile ConcurrentMap<String, Object> lookup;
    private volatile Set<String> keyset;
    private final AtomicMarkableReference<Object[][]> parallelContents;
    
    protected ParallelListResourceBundle() {
        this.parallelContents = new AtomicMarkableReference(null, false);
    }
    
    protected abstract Object[][] getContents();
    
    ResourceBundle getParent() {
        return this.parent;
    }
    
    public void setParallelContents(OpenListResourceBundle openListResourceBundle) {
        if (openListResourceBundle == null) {
            this.parallelContents.compareAndSet(null, null, false, true);
        }
        else {
            this.parallelContents.compareAndSet(null, openListResourceBundle.getContents(), false, false);
        }
    }
    
    boolean areParallelContentsComplete() {
        if (this.parallelContents.isMarked()) {
            return true;
        }
        boolean[] markHolder = { false };
        return (Object[][])this.parallelContents.get(markHolder) != null || markHolder[0];
    }
    
    @Override
    protected Object handleGetObject(String s) {
        if (s == null) {
            throw new NullPointerException();
        }
        this.loadLookupTablesIfNecessary();
        return this.lookup.get(s);
    }
    
    @Override
    public Enumeration<String> getKeys() {
        return Collections.enumeration(this.keySet());
    }
    
    @Override
    public boolean containsKey(String s) {
        return this.keySet().contains(s);
    }
    
    @Override
    protected Set<String> handleKeySet() {
        this.loadLookupTablesIfNecessary();
        return this.lookup.keySet();
    }
    
    @Override
    public Set<String> keySet() {
        Set<Object> keyset;
        while ((keyset = this.keyset) == null) {
            ParallelListResourceBundle.KeySet keyset2 = new ParallelListResourceBundle.KeySet(this.handleKeySet(), this.parent);
            synchronized (this) {
                if (this.keyset == null) {
                    this.keyset = keyset2;
                }
                else {
                    continue;
                }
            }
        }
        return keyset;
    }
    
    synchronized void resetKeySet() {
        this.keyset = null;
    }
    
    void loadLookupTablesIfNecessary() {
        ConcurrentMap<Object, Object> lookup = this.lookup;
        if (lookup == null) {
            lookup = new ConcurrentHashMap();
            for (Object[] array : this.getContents()) {
                lookup.put((Object)array[0], array[1]);
            }
        }
        Object[][] array2 = (Object[][])this.parallelContents.getReference();
        if (array2 != null) {
            for (Object[] array3 : array2) {
                lookup.putIfAbsent((Object)array3[0], array3[1]);
            }
            this.parallelContents.set(null, true);
        }
        if (this.lookup == null) {
            synchronized (this) {
                if (this.lookup == null) {
                    this.lookup = lookup;
                }
            }
        }
    }
    
    private static class KeySet extends AbstractSet<String>
    {
        private final Set<String> set;
        private final ResourceBundle parent;
        
        private KeySet(Set<String> set, ResourceBundle parent) {
            this.set = set;
            this.parent = parent;
        }
        
        @Override
        public boolean contains(Object o) {
            return this.set.contains(o) || (this.parent != null && this.parent.containsKey((String)o));
        }
        
        @Override
        public Iterator<String> iterator() {
            if (this.parent == null) {
                return this.set.iterator();
            }
            return new Iterator<String>() {
                private Iterator<String> itr = ParallelListResourceBundle.KeySet.this.set.iterator();
                private boolean usingParent;
                
                @Override
                public boolean hasNext() {
                    if (this.itr.hasNext()) {
                        return true;
                    }
                    if (!this.usingParent) {
                        HashSet set = new HashSet(ParallelListResourceBundle.KeySet.this.parent.keySet());
                        set.removeAll(ParallelListResourceBundle.KeySet.this.set);
                        this.itr = set.iterator();
                        this.usingParent = true;
                    }
                    return this.itr.hasNext();
                }
                
                @Override
                public String next() {
                    if (this.hasNext()) {
                        return (String)this.itr.next();
                    }
                    throw new NoSuchElementException();
                }
                
                @Override
                public void remove() {
                    throw new UnsupportedOperationException();
                }
            };
        }
        
        @Override
        public int size() {
            if (this.parent == null) {
                return this.set.size();
            }
            HashSet set = new HashSet(this.set);
            set.addAll(this.parent.keySet());
            return set.size();
        }
    }
}
